import PopupManager from '@opentiny/vue-renderless/common/deps/popup-manager';
import { addClass, getStyle, removeClass } from '@opentiny/vue-renderless/common/deps/dom';
import { $prefix, setup, createComponent, hooks, directive as directive$1, setupComponent } from '@opentiny/vue-common';
import { renderless, api } from '@opentiny/vue-renderless/loading/vue';
import { createElementVNode, openBlock, createBlock, Transition, withCtx, withDirectives, normalizeClass, normalizeStyle, createElementBlock, resolveDynamicComponent, toDisplayString, createCommentVNode, vShow } from 'vue';
import afterLeave from '@opentiny/vue-renderless/common/deps/after-leave';
import '@opentiny/vue-theme/loading/index.css';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

var script = {
  inheritAttrs: false,
  name: $prefix + 'Loading',
  props: {
    _constants: Object
  },
  setup: function setup$1(props, context) {
    return setup({
      props: props,
      context: context,
      renderless: renderless,
      api: api,
      mono: true
    });
  }
};

var _hoisted_1 = {
  class: "tiny-loading__spinner"
};
var _hoisted_2 = {
  key: 0,
  class: "circular",
  viewBox: "25 25 50 50"
};
var _hoisted_3 = createElementVNode("circle", {
  class: "path",
  cx: "50",
  cy: "50",
  r: "24",
  fill: "none"
}, null, -1);
var _hoisted_4 = [_hoisted_3];
var _hoisted_5 = {
  key: 2,
  class: "tiny-loading__text"
};
function render(_ctx, _cache, $props, $setup, $data, $options) {
  return openBlock(), createBlock(Transition, {
    name: "tiny-loading-fade",
    onAfterLeave: _ctx.handleAfterLeave
  }, {
    default: withCtx(function () {
      return [withDirectives(createElementVNode("div", {
        class: normalizeClass(["tiny-loading__mask", [_ctx.state.customClass, {
          'is-fullscreen': _ctx.state.fullscreen
        }]]),
        style: normalizeStyle({
          backgroundColor: _ctx.state.background || ''
        })
      }, [createElementVNode("div", _hoisted_1, [!_ctx.state.spinner ? (openBlock(), createElementBlock("svg", _hoisted_2, _hoisted_4)) : (openBlock(), createBlock(resolveDynamicComponent(_ctx.state.spinner), {
        key: 1,
        class: "tiny-icon-loading"
      })), _ctx.state.text ? (openBlock(), createElementBlock("p", _hoisted_5, toDisplayString(_ctx.state.text), 1)) : createCommentVNode("", true)])], 6), [[vShow, _ctx.state.visible]])];
    }),
    _: 1
  }, 8, ["onAfterLeave"]);
}

script.render = render;

var defaults = {
  text: null,
  body: false,
  lock: false,
  customClass: '',
  fullscreen: true
};
var fullscreenLoading = null;
var constants = {
  TEXT_ATTR: 'tiny-loading__text',
  IS_FULLSCREEN_CLS: 'is-fullscreen',
  TEXT_SPINNER: 'tiny-loading__spinner',
  TEXT_BACKGROUND: 'tiny-loading__background',
  TEXT_CUSTOM_CLS: 'tiny-loading__custom-class',
  PARENT_HIDDEN_CLS: 'tiny-loading__parent-hidden',
  PARENT_RELATIVE_CLS: 'tiny-loading__parent-relative'
};
var addStyle = function addStyle(options, parent, instance) {
  var maskStyle = {};
  if (options.fullscreen) {
    instance.originalPosition = getStyle(document.body, 'position');
    instance.originalOverflow = getStyle(document.body, 'overflow');
    maskStyle.zIndex = PopupManager.nextZIndex();
  } else if (options.body) {
    var clientRect = options.target.getBoundingClientRect();
    instance.originalPosition = getStyle(document.body, 'position');
    var direction = ['top', 'left'];
    direction.forEach(function (property) {
      var scroll = property === 'top' ? 'scrollTop' : 'scrollLeft';
      maskStyle[property] = clientRect[property] + document.body[scroll] + document.documentElement[scroll] + 'px';
    });
    var size = ['height', 'width'];
    size.forEach(function (property) {
      maskStyle[property] = clientRect[property] + 'px';
    });
  } else {
    instance.originalPosition = getStyle(parent, 'position');
  }
  Object.keys(maskStyle).forEach(function (property) {
    instance.$el.style[property] = maskStyle[property];
  });
};
var service = (function () {
  var configs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  configs = _objectSpread2(_objectSpread2({}, defaults), configs);
  if (typeof configs.target === 'string') {
    configs.target = document.querySelector(configs.target);
  }
  configs.target = configs.target || document.body;
  if (configs.target !== document.body) {
    configs.fullscreen = false;
  } else {
    configs.body = true;
  }
  if (configs.fullscreen && fullscreenLoading && !fullscreenLoading.state.closed) {
    return fullscreenLoading;
  }
  var parent = configs.body ? document.body : configs.target;
  var instance = createComponent({
    component: script,
    propsData: {
      _constants: constants
    },
    el: document.createElement('div')
  });
  for (var key in configs) {
    if (Object.prototype.hasOwnProperty.call(configs, key)) {
      instance.state[key] = configs[key];
    }
  }
  addStyle(configs, parent, instance);
  if (instance.originalPosition !== 'absolute' && instance.originalPosition !== 'fixed') {
    addClass(parent, constants.PARENT_RELATIVE_CLS);
  }
  if (configs.fullscreen && configs.lock) {
    addClass(parent, constants.PARENT_HIDDEN_CLS);
  }
  parent.appendChild(instance.$el);
  hooks.nextTick(function () {
    instance.state.visible = true;
  });
  if (configs.fullscreen) {
    fullscreenLoading = instance;
  }
  return instance;
});

var insertDom = function insertDom(parent, el, binding) {
  if (!el.domVisible && getStyle(el, 'display') !== 'none' && getStyle(el, 'visibility') !== 'hidden') {
    Object.keys(el.maskStyle).forEach(function (property) {
      el.mask.style[property] = el.maskStyle[property];
    });
    if (el.originalPosition !== 'absolute' && el.originalPosition !== 'fixed') {
      addClass(parent, constants.PARENT_RELATIVE_CLS);
    }
    if (binding.modifiers.fullscreen && binding.modifiers.lock) {
      addClass(parent, constants.PARENT_HIDDEN_CLS);
    }
    el.domVisible = true;
    parent.appendChild(el.mask);
    hooks.nextTick(function () {
      if (el.instance.hiding) {
        el.instance.$emit('after-leave');
      } else {
        el.instance.state.visible = true;
      }
    });
    el.domInserted = true;
  } else if (el.domVisible && el.instance.hiding === true) {
    el.instance.state.visible = true;
    el.instance.hiding = false;
  }
};
var appendLoadingToBody = function appendLoadingToBody(el, binding) {
  var clientRect = el.getBoundingClientRect();
  el.originalPosition = getStyle(document.body, 'position');
  var direction = ['top', 'left'];
  direction.forEach(function (property) {
    var scroll = property === 'top' ? 'scrollTop' : 'scrollLeft';
    el.maskStyle[property] = clientRect[property] + document.body[scroll] + document.documentElement[scroll] - parseInt(getStyle(document.body, "margin-".concat(property)), 10) + 'px';
  });
  var size = ['height', 'width'];
  size.forEach(function (property) {
    el.maskStyle[property] = clientRect[property] + 'px';
  });
  insertDom(document.body, el, binding);
};
var toggleLoading = function toggleLoading(el, binding) {
  if (binding.value) {
    hooks.nextTick(function () {
      if (binding.modifiers.fullscreen) {
        el.originalPosition = getStyle(document.body, 'position');
        el.originalOverflow = getStyle(document.body, 'overflow');
        el.maskStyle.zIndex = PopupManager.nextZIndex();
        addClass(el.mask, constants.IS_FULLSCREEN_CLS);
        insertDom(document.body, el, binding);
      } else {
        removeClass(el.mask, constants.IS_FULLSCREEN_CLS);
        if (binding.modifiers.body) {
          appendLoadingToBody(el, binding);
        } else {
          el.originalPosition = getStyle(el, 'position');
          insertDom(el, el, binding);
        }
      }
    });
  } else {
    afterLeave(el.instance, function () {
      if (!el.instance.hiding) {
        return;
      }
      var target = binding.modifiers.fullscreen || binding.modifiers.body ? document.body : el;
      el.domVisible = false;
      removeClass(target, constants.PARENT_RELATIVE_CLS);
      removeClass(target, constants.PARENT_HIDDEN_CLS);
      el.instance.hiding = false;
    }, 300, true);
    el.instance.state.visible = false;
    el.instance.hiding = true;
  }
};
var vLoading = {
  bind: function bind(el, binding, vnode) {
    var vm = vnode.context;
    var textExr = el.getAttribute(constants.TEXT_ATTR);
    var spinnerExr = el.getAttribute(constants.TEXT_SPINNER);
    var backgroundExr = el.getAttribute(constants.TEXT_BACKGROUND);
    var customClassExr = el.getAttribute(constants.TEXT_CUSTOM_CLS);
    var mask = createComponent({
      component: script,
      propsData: {
        _constants: constants
      },
      el: document.createElement('div')
    });
    var config = {
      text: vm && vm[textExr] || textExr,
      spinner: vm && vm[spinnerExr] || spinnerExr,
      background: vm && vm[backgroundExr] || backgroundExr,
      customClass: vm && vm[customClassExr] || customClassExr,
      fullscreen: !!binding.modifiers.fullscreen
    };
    for (var key in config) {
      if (Object.prototype.hasOwnProperty.call(config, key)) {
        mask.state[key] = config[key];
      }
    }
    el.instance = mask;
    el.mask = mask.$el;
    el.maskStyle = {};
    binding.value && toggleLoading(el, binding);
  },
  update: function update(el, binding) {
    el.instance.setText(el.getAttribute(constants.TEXT_ATTR));
    if (binding.oldValue !== binding.value) {
      toggleLoading(el, binding);
    }
  },
  unbind: function unbind(el, binding) {
    if (el.domInserted) {
      el.mask && el.mask.parentNode && el.mask.parentNode.removeChild(el.mask);
      toggleLoading(el, {
        value: false,
        modifiers: binding.modifiers
      });
    }
  }
};
var directive = directive$1({
  vLoading: vLoading
}).vLoading;

var Loadings = {};
Loadings.install = function (app) {
  app.directive('loading', directive);
};
setupComponent.TINYLoading = {
  init: function init(root) {
    root.$loading = service;
  }
};
Loadings.service = service;
Loadings.directive = directive;
Loadings.version = '3.1.0-alpha.0';

export { Loadings as default };
